from cStringIO import StringIO
from Products.KBannerAd.config import *
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal

def addCatalogIndexes (self, out):
    # add indexes and metadatas to the portal catalog
    ct = getToolByName(self, 'portal_catalog')
    for idx in catalog_indexes:
        if idx['name'] in ct.indexes():
            out.write("Found the '%s' index in the catalog, nothing changed.\n" % idx['name'])
        else:
            ct.addIndex(**idx)
            out.write("Added '%s' (%s) to the catalog.\n" % (idx['name'], idx['type']))
    for entry in catalog_metadata:
        if entry in ct.schema():
            out.write("Found '%s' in the catalog metadatas, nothing changed.\n" % entry)
        else:
            ct.addColumn(entry)
            out.write("Added '%s' to the catalog metadatas.\n" % entry)
    

def delCatalogIndexes (self, out):
    # remove indexes and metadatas to the portal catalog
    ct = getToolByName(self, 'portal_catalog')
    for idx in catalog_indexes:
        if idx['name'] in ct.indexes():
            try:
                ct.delIndex(idx['name'])
                out.write("Removed the '%s' index from the catalog.\n" % idx['name'])
            except:
                pass
    for entry in catalog_metadata:
        if entry in ct.schema():
            try:
                ct.delColumn(entry)
                out.write("Removed '%s' from the catalog metadatas.\n" % entry)
            except:
                pass

def install(self):
  out = StringIO()

  # customize Page
  portal_object = getToolByName(self, 'portal_url').getPortalObject()
  
  #custom view methods
  for type in VIEW_METHODS:
    out.write('+ Installing custom views for %s\n' % type)
    
    views = VIEW_METHODS.get(type)
    if views:
      custom_type = getattr(portal_object.portal_types, type)
      if custom_type:
        setattr(custom_type, 'default_view', views[0])
        setattr(custom_type, 'immediate_view', views[0])
        setattr(custom_type, 'view_methods', views)
  
  # disable actions
  for action_provider in ACTIONS:
    
    changed_actions = ACTIONS[action_provider]
    
    mt = getToolByName(self, action_provider)
    _actions=mt._cloneActions()
  
    for action in _actions:
      
      for changed_action in changed_actions:
        
        if action.id == changed_action['id']:
          
          for attribute in changed_action:
            setattr(action, attribute, changed_action[attribute])
            out.write('+ set attribute %s to %s on action %s\n' % (attribute, changed_action[attribute], action.id))
          
    mt._actions=_actions

  mt = getToolByName(self, 'portal_actions')
  for action in ADD_ACTIONS:
    try: 
      mt.addAction(action['id'],
                   name=action.get('name', ''),
                   action=action.get('action',''),
                   condition=action.get('condition',''),
                   permission=action.get('permission', 'View'),
                   category=action['category'])
      out.write('+ added action %s to portal_actions.\n' % action['id'])
    except:
      pass
  
  addCatalogIndexes(self, out)
  
  out.write("Installation completed.")
  return out.getvalue()
  
def uninstall (self):
    out = StringIO ()

    delCatalogIndexes(self, out)
    
    out.write ('Uninstallation completed.\n')
    return out.getvalue()